import unittest
from fabfile import clean_test
from fabric.api import local


from fab_support import set_stages, copy_null, copy_file


class TestBasicFabSupport(unittest.TestCase):

    def setUp(self):
        # local('pelican-quickstart')
        pass

    def tearDown(self):
        clean_test()

    def test_set_stages(self):
        # Can you call it
        # Definition of different environments to deploy to
        set_stages (globals(), {
            'testhost': {
                'comment': 'stage: Local build and serving from output directory',
                'config_file': 'local_conf.py',
                'destination': '',
                'copy_method': copy_null,
                'SITEURL': 'http://localhost:8000',
                'PORT': 8000,
            },
        })
        local('pelican ')




"""
Thanks https://stackoverflow.com/questions/13761697/how-to-access-the-unittest-mainverbosity-setting-in-a-unittest-testcase
I can't believe this is the simplest way.
"""
import inspect
import unittest

def unittest_verbosity():
    """Return the verbosity setting of the currently running unittest
    program, or 0 if none is running.

    """
    frame = inspect.currentframe()
    while frame:
        self = frame.f_locals.get('self')
        if isinstance(self, unittest.TestProgram):
            return self.verbosity
        frame = frame.f_back
    return 0

def verbose():
    return True  # TODO debug unittest_verbosity() > 2



# -*- coding: utf-8 -*-
"""stages_support
These are functions
"""
from fabric.api import env, task


@task
def list_stages():
    print('Hi from list_stages')
    try:
        stages = env['stages']
        print('List of stages')
        print(stages)
        for stage_name, stage in stages.items():
            try:
                comment = stage['comment']
            except KeyError:
                comment = ''
            print(f'{stage_name} - {comment}')
    except KeyError:
        for k, v in env:
            if k.lower() == 'stages':
                print("env['{f}'] has been set but should probably be 'stages'")
        print("env['stages'] has not been set.")


__all__ = []



from django.db import models


class MyData(models.Model):
    name = models.CharField(verbose_name='A Name', blank=True, max_length=255)
    number = models.IntegerField(verbose_name='A Number')

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = 'MyData'



# Generated by Django 2.0.2 on 2018-02-14 20:09

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, verbose_name='A Name')),
                ('number', models.IntegerField(verbose_name='A Number')),
            ],
        ),
    ]



import os
import shutil
import unittest

from fabric.api import local, lcd

from tests.utils import verbose


def clean_setup():
    # Try and remove running apps however relies on demo_django directory not being deleted
    # before try and remove
    if os.path.isdir('tests'):
        my_path = 'tests/demo_django'
        my_path2 = 'tests/'
    elif os.path.isdir('template_files'):
        my_path = 'demo_django'
        my_path2 = ''
    else:
        raise Exception
    with lcd(my_path):
        try:
            local('fab fab_support.django.kill_app:demo')  # Remove any existing run time
        except SystemExit:
            pass
    # bodge to get .git file delete keeps giving a PermissionError after testing about a git file
    try:
        with lcd(my_path):
            local('del /F /S /Q /A .git')
            pass
    except (FileNotFoundError, SystemExit):
        pass
    with lcd(my_path2):
        try:
            shutil.rmtree(my_path2+'demo_django')
        except FileNotFoundError:
            result = local('dir', capture=True)
            print('== Failed to remove tree ==')
            print(result)


def clean_test_django():
    clean_setup()
    print('Cleaned test_django extras')


class TestBasicFabSupport(unittest.TestCase):

    def setUp(self):
        clean_setup()
        local('django-admin startproject demo_django')
        # TODO convert paths to windows and linux
        local('copy template_files\\demo_django_fabfile.py demo_django\\fabfile.py')
        local('copy template_files\\demo_django.env demo_django\\.env')  # Need to have an environment for secrets
        local('copy template_files\\demo_django_Procfile demo_django\\Procfile')  # Need to a Procfile to tell
        # heroku what types of workers you need
        local('copy template_files\\demo_requirements.txt demo_django\\requirements.txt')  # Need to tell Heroku that
        # this is a python project and what is needed in production
        local(
            'copy template_files\\demo_django_settings.py demo_django\\demo_django\\settings.py /Y')  # Need to customise
        # for collect statics (alternative would be to ignore collect static)
        # Setup a git for Heroku to use to deploy demo_django
        local('git init demo_django')
        with lcd('demo_django'):
            local('mkdir static')  # Heroku needs a place to put static files in collectstatic and won't create it.
            local('copy ..\\template_files\\demo_django_fabfile.py static\\fabfile.py')  # To make git recognise it
            local('git add .')
            local("git commit -m 'start'")

    def tearDown(self):
        pass

    def test_list(self):
        """Aim is to run fab --list making sure we are running fab in the correct test directory"""
        with lcd('demo_django'):
            # Check staging and fabfile are correct
            result = local('fab --list', capture=True)
            if verbose():
                print(result)
            self.assertNotRegex(result, 'builds source and wheel', 'Should not be using main fabfile')
            self.assertRegex(result, 'test_demo_django_fab_file', 'should be using local fab file with local task')
            self.assertRegex(result, 'list_stages', 'Should have task list_stages')

    def test_list_stages(self):
        """Should have one and only demo when you list stages"""
        with lcd('demo_django'):
            # Check staging and fabfile are correct
            result = local('fab fab_support.list_stages', capture=True)
            if verbose():
                print(result)
            self.assertRegex(result, 'Demo version of Django', 'demo stage')

    def test_got_heroku_and_build(self):
        """
        The Django version is meant to be as simple as possible. eg it is running from a sqlite database even
        though it has a Postgres Database available.
        After running this the directory is kept available.  You can test the server locally by:
        - running the dev environment
        - switching to tests\demo_django
        - run `python manage.py runserver`
        - open localhost:8000

        The Heroku test version that is spun up is a test version at zero cost.
        This should be made into another functional test.
        """
        with lcd('demo_django'):
            # Check staging and fabfile are correct
            result = local('fab --list', capture=True)
            self.assertRegex(result, 'demo', 'demo stage')
            self.assertRegex(result, 'test_demo_django_fab_file', 'The fabfile has defined a new task.')
            try:
                local('fab fab_support.django.kill_app:demo')  # Remove any existing run time
            except SystemExit:
                pass
            local('fab fab_support.django.create_newbuild:demo')  # Build database from scratch
            # TODO add selenium to check that website is up




import datetime as dt
from fabric.api import env, local, task, lcd
from fabric.operations import require
import json
import os
import re
import time

DJANGO_SETTINGS_MODULE = 'production'  # Which settings configuration to use in Django
HEROKU_APP_NAME = 'fab-support-test-app'
HEROKU_POSTGRES_TYPE = 'hobby-dev'
SUPERUSER_NAME = 'superuser'  # TODO may want to get rid of defaults
SUPERUSER_EMAIL = 'info@demo.com'
# noinspection SpellCheckingInspection
SUPERUSER_PASSWORD = 'akiualsdfha*&(j'  # TODO Need alternative
GIT_PUSH = ''  # Default to false
GIT_PUSH_DIR = '.'  #
GIT_BRANCH = 'master'
USES_CELERY = False


##################################################
# Local utilities
##################################################

def remove_unused_db(env_prefix='uat'):
    """List all databases in use for app, find the main one and remove all the others"""
    heroku_app = '{0}-{1}'.format(os.environ['HEROKU_PREFIX'], env_prefix)
    data = json.loads(local(f'heroku config --json --app {heroku_app}', capture=True))
    for k, v in data.items():
        # noinspection SpellCheckingInspection
        if k.find('HEROKU_POSTGRESQL_') == 0:
            if v != data['DATABASE_URL']:
                local(f'heroku addons:destroy {k} --app {heroku_app} --confirm {heroku_app}')


def default_db_colour(app_name):
    """Return the default database colour of heroku application"""
    data = json.loads(local('heroku config --json --app {0}'.format(app_name), capture=True))
    result = ''
    for k, v in data.items():
        if k.find('HEROKU_POSTGRESQL_') == 0:
            if v == data['DATABASE_URL']:
                return k
    # if no colour found then try the long name in database_url
    # raise Exception(f'No color database names found for app {app_name} - create an extra one and it should be ok.')
    return data['DATABASE_URL']


def set_heroku_environment_variables():
    """This sets all the environment variables that a Django recipe needs."""
    local(f"heroku config:set DJANGO_SETTINGS_MODULE={DJANGO_SETTINGS_MODULE} --app {HEROKU_APP_NAME}")
    local(f"heroku config:set PYTHONHASHSEED=random --app {HEROKU_APP_NAME}")
    try:
        allowed_hosts = os.environ['DJANGO_ALLOWED_HOSTS']
    except KeyError:
        allowed_hosts = f'{HEROKU_APP_NAME}.herokuapp.com'
    local(f'heroku config:set DJANGO_ALLOWED_HOSTS="{allowed_hosts}" --app {HEROKU_APP_NAME}')
    # TODO 'DJANGO_SECRET_KEY' needs to be installed or have useful defaults
    not_used = []
    for config in ('DJANGO_SECRET_KEY', 'DJANGO_ADMIN_URL'
                   , 'DJANGO_AWS_ACCESS_KEY_ID', 'DJANGO_AWS_SECRET_ACCESS_KEY', 'DJANGO_AWS_STORAGE_BUCKET_NAME'
                   , 'DJANGO_MAILGUN_API_KEY', 'DJANGO_SERVER_EMAIL', 'MAILGUN_SENDER_DOMAIN'
                   , 'DJANGO_ACCOUNT_ALLOW_REGISTRATION', 'DJANGO_SENTRY_DSN'
                   , 'XERO_CONSUMER_SECRET', 'XERO_CONSUMER_KEY'):
        try:
            local('heroku config:set {}={} --app {}'.format(config, os.environ[config], HEROKU_APP_NAME))
        except KeyError:
            # This environment variable won't be set
            not_used.append(config)
    if not_used:
        print(f'The following config variables were not used: {not_used}')


def raw_update_app():
    """Update of app to latest version"""
    # Put the heroku app in maintenance mode TODO
    set_heroku_environment_variables()  # In case anything has changed
    # connect git to the correct remote repository
    local('heroku git:remote -a {}'.format(HEROKU_APP_NAME))
    # Need to push the branch in git to the master branch in the remote heroku repository
    print(f'GIT_PUSH_DIR = {GIT_PUSH_DIR}, GIT_PUSH = {GIT_PUSH}, GIT_BRANCH = {GIT_BRANCH}')
    if GIT_PUSH == '':  # test for special case probably deploying a subtree
        local(f'git push heroku {GIT_BRANCH}:master')
    else:
        # The command will probably be like this:
        # 'GIT_PUSH': 'git subtree push --prefix tests/my_heroku_project heroku master',
        with lcd(GIT_PUSH_DIR):
            local(GIT_PUSH)
    # Don't need to scale workers down as not using eg heroku ps:scale worker=0
    # Will add guvscale to spin workers up and down from 0
    if USES_CELERY:
        local(f'heroku ps:scale worker=1 -a {HEROKU_APP_NAME}')
    # Have used performance web=standard-1x and worker=standard-2x but adjusted app to used less memory
    # local(f'heroku ps:resize web=standard-1x -a {heroku_app}')  # Resize web to be compatible with performance workers
    # local(f'heroku ps:resize worker=standard-2x -a {heroku_app}')  # Resize workers
    # makemigrations should be run locally and the results checked into git
    local('heroku run "yes \'yes\' | python manage.py migrate"')  # Force deletion of stale content types


def install_heroku_plugins(plug_in_list):
    # plugins doesn't support returning --json
    results = local('heroku plugins --core', capture=True)  # returns string or string list
    result_list = results.split('\n')
    plugin_dict = {}
    for result in result_list:
        parts = result.split(' ')
        try:
            plugin_dict[parts[0]] = parts[1]
        except IndexError:
            plugin_dict[parts[0]] = ''
    for plug_in in plug_in_list:
        if not plug_in in plugin_dict:
            local(f'heroku plugins:install {plug_in}')  # installed in local toolbelt not on app
            # If it fails then it really is a failure not just it has already been installed.
    return True  # Got to end and all installed
    # print(f'|{results}|')


# #############
def _create_newbuild():
    """This builds the database and waits for it be ready.  It is is safe to run and won't
    destroy any existing infrastructure."""
    local(f'heroku create {HEROKU_APP_NAME} --buildpack https://github.com/heroku/heroku-buildpack-python --region eu')
    # This is where we create the database.  The type of database can range from hobby-dev for small
    # free access to standard for production quality docs
    local(f'heroku addons:create heroku-postgresql:{HEROKU_POSTGRES_TYPE} --app {HEROKU_APP_NAME}')
    local(f'heroku addons:create cloudamqp:lemur --app {HEROKU_APP_NAME}')
    local(f'heroku addons:create papertrail:choklad --app {HEROKU_APP_NAME}')
    # Add guvscale processing to allow celery queue to be at zero
    install_heroku_plugins(['heroku-cli-oauth', 'heroku-guvscale'])
    # start of configuring guvscale to autoscale
    # local(f'heroku guvscale:getconfig --app {heroku_app}')
    # set database backup schedule
    local(f'heroku pg:wait --app {HEROKU_APP_NAME}')  # It takes some time for DB so wait for it
    local(f'heroku pg:backups:schedule --at 04:00 --app {HEROKU_APP_NAME}')
    # Already promoted as new local('heroku pg:promote DATABASE_URL --app my-app-prod')
    # Leaving out and aws and reddis
    raw_update_app()
    local('heroku run python manage.py check --deploy')  # make sure all ok

    # Create superuser - the interactive command does not allow you to script the password
    # So this is a hack  workaround.
    # Django 1 only
    # cmd = ('heroku run "echo \'from django.contrib.auth import get_user_model; User = get_user_model(); '
    #       + f'User.objects.filter(email="""{SUPERUSER_EMAIL}""", is_superuser=True).delete(); '
    #       + f'User.objects.create_superuser("""{SUPERUSER_NAME}""", """{SUPERUSER_EMAIL}""", """{SUPERUSER_PASSWORD}""")\' '
    #       + f' | python manage.py shell"')
    # local(cmd)


def get_global_environment_variables(stage):
    # Get a number of predefined environment variables from the staging system variables
    # and turn them into globals for use in this script
    # TODO perhaps cconvert to another method of access
    for global_env in ('HEROKU_APP_NAME', 'HEROKU_POSTGRES_TYPE', 'SUPERUSER_NAME', 'USES_CELERY',
                       'SUPERUSER_EMAIL', 'SUPERUSER_PASSWORD',
                       'GIT_BRANCH', 'GIT_PUSH', 'GIT_PUSH_DIR',
                       'DJANGO_SETTINGS_MODULE'):
        try:
            globals()[global_env] = env['stages'][stage][global_env]
        except KeyError:
            # This global variable will use the default
            pass


@task
def create_newbuild(stage):
    get_global_environment_variables(stage)
    _create_newbuild()


def is_production():
    return HEROKU_APP_NAME[-4:].lower() == 'prod'


def _kill_app():
    """see kill app"""
    local(f'heroku destroy {HEROKU_APP_NAME} --confirm {HEROKU_APP_NAME}')


@task
def kill_app(stage, safety_on=True):
    """Kill app notice that to the syntax for the production version is:
    fab the_stage kill_app:False"""
    # Todo Add steps to verify that it exists (optional) and make sure it is deleted at the end
    if not (is_production() and not safety_on):
        get_global_environment_variables(stage)
        _kill_app()


@task
def build_uat():
    """Build a new uat environments"""
    build_app()


@task
def build_app(stage='uat'):
    """Build a test environment. Default is uat.
    So fab build_app  is equivalent to fab build_app:uat  and to fab build_app:env_prefix=uat
    so can build a test branch with:
        fab build_app:env_prefix=test"""
    start_time = time.time()
    try:
        _kill_app()
    except SystemExit:
        if stage != 'prod':
            pass  # ignore errors in case original does not exist
        else:
            raise Exception('Must stop if an error when deleting a production database.')
    _create_newbuild()
    local(f'fab transfer_database_from_production:{env_prefix}')
    # makemigrations should be run locally and the results checked into git
    # Need to migrate the old database schema from the master production database
    local('heroku run "yes \'yes\' | python manage.py migrate"')  # Force deletion of stale content types
    # Calculate time
    end_time = time.time()
    runtime = str(dt.timedelta(seconds=int(end_time - start_time)))
    print(f'Run time = {runtime} Completed at: {dt.datetime.now()}')


@task
def create_new_db(stage='uat'):
    """Just creates a new database for this instance."""
    heroku_app = '{0}-{1}'.format(os.environ['HEROKU_PREFIX'], stage)
    # Put the heroku app in maintenance move
    m = local('heroku addons:create heroku-postgresql:hobby-basic --app {0}'.format(heroku_app), capture=True)
    m1 = m.replace('\n',' ')  # Convert to a single string
    print(f'>>>{m1}<<<')
    found = re.search('Created\w*(.*)\w*as\w*(.*)\w* Use', m1)
    db_name = found.group(1)
    colour = found.group(2)
    print(f'DB colour = {colour}, {db_name}')
    local('heroku pg:wait')  # It takes some time for DB so wait for it
    return colour, db_name


def _transfer_database_from_production(stage='test', clean=True):
    """This is usually used for making a copy of the production database for a UAT staging
    or test environment.  It can also be used to upgrade the production environment from one
    database plan to the next. """
    heroku_app = '{0}-{1}'.format(os.environ['HEROKU_PREFIX'], stage)
    heroku_app_prod = '{0}-prod'.format(os.environ['HEROKU_PREFIX'])
    # Put the heroku app in maintenance move
    try:
        local('heroku maintenance:on --app {} '.format(heroku_app))
        colour, db_name = create_new_db(stage)  # color is ?
        # Don't need to scale workers down as not using eg heroku ps:scale worker=0
        local(f'heroku pg:copy {heroku_app_prod}::DATABASE_URL {colour} --app {heroku_app} --confirm {heroku_app}')
        local(f'heroku pg:promote {colour}')
        if clean:
            remove_unused_db(stage)
    finally:
        local('heroku maintenance:off --app {} '.format(heroku_app))


@task
def transfer_database_from_production(env_prefix='test', clean=True):
    _transfer_database_from_production(env_prefix, clean)


@task
def list_stages():
    """This is put here to test the exact same code in django as in set_stages.  In  one it seems to work
    and another to fail."""
    print('Hi from list_stages in django')
    try:
        stages = env['stages']
        print('List of stages')
        print(stages)
        for stage_name, stage in stages.items():
            try:
                comment = stage['comment']
            except KeyError:
                comment = ''
            print(f'{stage_name} - {comment}')
    except KeyError:
        for k, v in env:
            if k.lower() == 'stages':
                print("env['{f}'] has been set but should probably be 'stages'")
        print("env['stages'] has not been set.")



"""
Django settings for demo_django project.

Generated by 'django-admin startproject' using Django 2.0.2.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
"""

import environ
import os

# Load operating system environment variables and then prepare to use them
env = environ.Env()

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
ROOT_DIR = environ.Path(__file__) - 2  # (demo_django/settings.py - 1 = demo_django_postgres/)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# .env file, should load only in development environment
# So in local environment set this variable -> set DJANGO_READ_DOT_ENV_FILE=True
READ_DOT_ENV_FILE = env.bool('DJANGO_READ_DOT_ENV_FILE', default=False)

if READ_DOT_ENV_FILE:
    # Operating System Environment variables have precedence over variables defined in the .env file,
    # that is to say variables from the .env files will only be used if not defined
    # as environment variables.
    env_file = str(ROOT_DIR.path('.env'))
    print('Loading : {}. See {} for more info'.format(env_file, __name__))
    env.read_env(env_file)


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'k+f@awoj58mtqdithnzz_c-nz@cx&j^edp0d#8$7mriv@8h_$6'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True


ALLOWED_HOSTS = os.getenv('DJANGO_ALLOWED_HOSTS', 'fab-support-test-app.herokuapp.com').split(',')


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.sites',
    'django.contrib.staticfiles',

    'django_tables2',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'allauth.socialaccount.providers.windowslive',

    'my_app.apps.MyAppConfig',

]

SITE_ID = 1  # for django allauth

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'demo_django.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': ['templates'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# AUTHENTICATION CONFIGURATION
# ------------------------------------------------------------------------------
AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
    'allauth.account.auth_backends.AuthenticationBackend',
]

WSGI_APPLICATION = 'demo_django.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases
# DATABASE CONFIGURATION
# ------------------------------------------------------------------------------

# Use the Heroku-style specification
# Raises ImproperlyConfigured exception if DATABASE_URL not in os.environ
DATABASES = {}
DATABASES['default'] = env.db('DATABASE_URL')



# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

# Static files (CSS, JavaScript, Images)
STATIC_ROOT = os.path.join(BASE_DIR, 'staticfiles')
STATIC_URL = '/static/'

# Extra places for collectstatic to find static files.
STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'static'),
)



import os
import unittest

from fabric.api import local, lcd

from tests.utils import verbose


def clean_setup_postgres():
    # Try and remove running apps however relies on demo_django directory not being deleted
    # before try and remove
    if os.path.isdir('tests'):
        my_path = 'tests/demo_django_postgres'
    elif os.path.isdir('template_files'):
        my_path = 'demo_django_postgres'
    else:
        raise Exception
    with lcd(my_path):
        for stage in ('test', 'uat', 'prod'):
            try:
                local(f'fab fab_support.django.kill_app:{stage}')  # Remove any existing run time
            except SystemExit:
                pass


class TestDjangoPostgresSupport(unittest.TestCase):

    def setUp(self):
        clean_setup_postgres()

    def tearDown(self):
        pass

    def test_list(self):
        """Aim is to run fab --list making sure we are running fab in the correct test directory"""
        with lcd('demo_django_postgres'):
            # Check staging and fabfile are correct
            result = local('fab --list', capture=True)
            if verbose():
                print(result)
            self.assertNotRegex(result, 'builds source and wheel', 'Should not be using main fabfile')
            self.assertRegex(result, 'test_django_postgres_fab_file', 'should be using local fab file with local task')
            self.assertRegex(result, 'list_stages', 'Should have task list_stages')

    def test_list_stages(self):
        """Should have one and only demo when you list stages"""
        with lcd('demo_django_postgres'):
            # Check staging and fabfile are correct
            result = local('fab fab_support.list_stages', capture=True)
            if verbose():
                print(result)
            self.assertRegex(result, 'Test version of Django', 'demo stage')
            self.assertRegex(result, 'uat', 'UAT stage')
            self.assertRegex(result, 'prod', 'production stage')

    def test_django_postgres(self):
        """
        The Django version is a basic Postgres application.  It has a single app with a single model.
        It is meant to examine the operation of Django app with test, uat and production builds.

        This allows test data and production data to be simulated.

        After running this the directory is kept available.  You can test the server locally as above.

        We will follow the following story:

        Build test with no data
        Build test with test data
        New Build UAT with new production data
        Promote UAT to production
        Build UAT with production data
        Update production
        Promote UAT to production
        Build test with production data


        The Heroku test version that is spun up is a test version at zero cost.
        """
        with lcd('demo_django_postgres'):
            local('fab fab_support.django.create_newbuild:test')  # Build database from scratch
            # local('fab demo fab_support.django.kill_app')  # By default don't let it run after test



from shutil import rmtree


def remove_tree(path):
    assert path not in ('c:\\', 'c:', '\\', '/')  # Add safety check
    if isinstance(path, tuple) or isinstance(path, list):
        for this in path:
            remove_tree(this)
    else:
        rmtree(path, ignore_errors=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `fab_support` package."""


import unittest

from fab_support import pelican


class TestFab_support(unittest.TestCase):
    """Tests for `fab_support` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""



try:
    from django.urls import reverse, reverse_lazy
except ImportError:
    from django.core.urlresolvers import reverse, reverse_lazy

from django.views.generic import ListView

from .models import MyData


class HomeView(ListView):
    template_name = 'name_list.html'
    # redirect_field_name = ''
    model = MyData

    def get_context_data(self, **kwargs):
        context = super(HomeView, self).get_context_data(**kwargs)
        context.update({'objects': MyData.objects.all(), 'version': 'Hi'})
        return context



from distutils.dir_util import copy_tree


class FabricSupportException(Exception):
    pass


# Utility functions
# Different methods of deployment
# copy_null eg for running locally from test directory
# noinspection PyUnusedLocal
def copy_null(source, destination):
    pass  # Already there


def copy_file(source, destination):
    copy_tree(source, destination)  # from one file location to another



# -*- coding: utf-8 -*-

"""Top-level package for fab_support."""

__author__ = """Humphrey Drummond"""
__email__ = 'hum3@drummond.info'
__version__ = '0.1.1'


from .env_support import *
from .stages_support import list_stages
from .utils import *




from fabric.api import task


import sys
from pathlib import Path  # if you haven't already done so
file = Path(__file__).resolve()
parent, root = file.parent, file.parents[2]
sys.path.append(str(root))

# noinspection PyUnresolvedReferences
import fab_support
from fab_support import set_stages
# noinspection PyUnresolvedReferences
from fab_support.django import kill_app, build_app  # Can call directly

# Definition of different environments to deploy to
set_stages(globals(), {
    'demo': {
        'comment': 'Demo version of Django to be deployed on Heroku',
        'DJANGO_SETTINGS_MODULE': 'demo_django.settings',  # Essential as django doesn't know name of app
        'HEROKU_APP_NAME': 'fab-support-test-demo',
        'GIT_BRANCH': 'master'
    },
})


@task
def test_demo_django_fab_file():
    """A test task to show correct file has been loaded"""
    pass



import unittest

from fabric.api import local, lcd

from tests.utils import verbose


class TestBasicFabSupport(unittest.TestCase):

    def test_list(self):
        """Aim is to run fab --list making sure we are running fab in the correct test directory"""
        with lcd('basic'):
            # Check staging and fabfile are correct
            result = local('fab --list', capture=True)
            if verbose():
                print(result)
            self.assertNotRegex(result, 'builds source and wheel', 'Should not be using main fabfile')
            self.assertRegex(result, 'test_demo_fab_file', 'should be using local fab file with local task')
            self.assertRegex(result, 'list_stages', 'Should have task list_stages')

    def test_list_stages(self):
        """Should have one and only demo when you list stages"""
        with lcd('basic'):
            # Check staging and fabfile are correct
            result = local('fab fab_support.list_stages', capture=True)
            if verbose():
                print(result)
            self.assertRegex(result, 'demo', 'demo stage')

    def test_list_stages2(self):
        """Should have one and only demo when you list stages"""
        with lcd('basic2'):
            # Check staging and fabfile are correct
            result = local('fab fab_support.list_stages', capture=True)
            if verbose():
                print(result)
            self.assertRegex(result, 'demo', 'demo stage')
            self.assertRegex(result, 'tag:456', 'production stage')



"""demo_django URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.conf.urls import include, url
from django.urls import path
from django.views.generic import TemplateView

from my_app.views import HomeView

urlpatterns = [
    path('admin/', admin.site.urls),
    url(r'^$',
        TemplateView.as_view(template_name='home.html'), name='home'),
    url(
        regex=r'^mydata$',
        view=HomeView.as_view(),
        name='mydata'
     ),
    url(r'^accounts/', include('allauth.urls')),
    # url(r'^about/$', TemplateView.as_view(template_name='pages/about.html'), name='about'),

    # Django Admin, use {% url 'admin:index' %}
    # url(settings.ADMIN_URL, admin.site.urls),


]  # + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# if settings.DEBUG:
#     # This allows the error pages to be debugged during development, just visit
#     # these url in browser to see how these error pages look like.
#     urlpatterns += [
#         url(r'^400/$', default_views.bad_request, kwargs={'exception': Exception('Bad Request!')}),
#         url(r'^403/$', default_views.permission_denied, kwargs={'exception': Exception('Permission Denied')}),
#         url(r'^404/$', default_views.page_not_found, kwargs={'exception': Exception('Page not Found')}),
#         url(r'^500/$', default_views.server_error),
#     ]
#     if 'debug_toolbar' in settings.INSTALLED_APPS:
#         import debug_toolbar
#         urlpatterns = [
#             url(r'^__debug__/', include(debug_toolbar.urls)),
#         ] + urlpatterns



from fabric.api import task, env

# Need to modify path so fab_support is found
# This is only needed to run test fabfile's in a test directory
import sys
from pathlib import Path  # if you haven't already done so
file = Path(__file__).resolve()
parent, root = file.parent, file.parents[2]
sys.path.append(str(root))

# noinspection PyUnresolvedReferences
import fab_support

env['stages'] = {
    'demo': {
        'comment': 'Simplest demo project',
    },
    'production': {
        'comment': 'Production project tag:456 for regex match',
    },
    }


@task
def test_demo_fab_file():
    """A test task to show correct file has been loaded"""
    pass



# -*- coding: utf-8 -*-
"""env_support
This supports using a .env environment file to store secrets in a file and then to make them available
to the local programs as environment variables.
"""


import datetime as dt
import os

from dotenv import load_dotenv, find_dotenv


# .Env
# Get any required secret environment variables
load_dotenv(find_dotenv())

os.environ['UPDATE_DATESTAMP'] = dt.datetime.now().isoformat()




import unittest

from fab_support.django import install_heroku_plugins

class TestHeroku(unittest.TestCase):

    def test_heroku(self):
        self.assertTrue(install_heroku_plugins([]), "install_heroku_plugins failed")
        self.assertTrue(install_heroku_plugins(['heroku-cli-oauth']), "install_heroku_plugins failed")



# -*- coding: utf-8 -*-
"""fab_support
"""


import boto3
import datetime as dt
from distutils.dir_util import copy_tree
import mimetypes
import os
import shutil
import sys
import socketserver

from fabric.api import run, local, env, settings, cd, task, put, execute
from fabric.operations import require
from unipath import Path, DIRS

from pelican.server import ComplexHTTPRequestHandler


@task
def build():
    """Build version of site ready for use locally or deployment to main site eg fab local build"""
    require('stage')
    copy_images()
    local(f'pelican -s {env.config_file}')


@task
def copy_images(alias='ci'):
    """Copy files in all images directories"""
    src = Path(env.deploy_path).ancestor(2).child('content')
    dst = src.ancestor(2).child('images')
    for my_dir in src.walk(filter=DIRS):  # os.walk('./..'):
        if my_dir.name == 'images':
            copy_tree(my_dir, dst)


@task
def rebuild():
    """`build` with the delete switch"""
    require('stage')
    local(f'pelican -d -s {env.config_file}')


@task
def regenerate():
    """Automatically regenerate site upon file modification"""
    require('stage')
    local('pelican -r -s {env.config_file}')


@task
def serve():
    """Serve site at eg http://localhost:8000/"""
    require('stage')
    os.chdir(env.deploy_path)

    class AddressReuseTCPServer(socketserver.TCPServer):
        allow_reuse_address = True

    server = AddressReuseTCPServer(('', env['PORT']), ComplexHTTPRequestHandler)

    sys.stderr.write('Serving on port {0} ...\n'.format(env['PORT']))
    server.serve_forever()


@task
def reserve():
    """`build`, then `serve`"""
    build()
    serve()


@task
def deploy():
    """Publish to stage. includes build."""
    require('stage')
    build()
    print(f'Using copy from {env.deploy_path} to {env.destination}')
    env['copy_method'](env.deploy_path, env.destination)
    # ? clean()
    # copy_tree(DEPLOY_PATH, LOCAL_SITE_PATH)
    # copy_tree(DEPLOY_PATH.ancestor(2).child(
    #     'Pelican').child('pelican-bootstrap3').child('static'),
    #                 LOCAL_SITE_PATH.child('static'))
    # copy_tree(DEPLOY_PATH.ancestor(1).child(
    #     'content').child('images'),
    #                 LOCAL_SITE_PATH.child('images'))


@task
def clean():
    """Remove newly generated files from remote environment"""
    require('stage')
    if os.path.isdir(env.destination):
        shutil.rmtree(env.destination)
        os.makedirs(env.destination)

        # ?try:
        #    shutil.rmtree(LOCAL_SITE_PATH)
        # except FileNotFoundError:
        #    pass  # doesn't exist


@task
def s3_upload():
    """Publish to S3"""
    rebuild()
    # get an access token, local (from) directory, and S3 (to) directory
    # from the command-line
    #local_directory = Path(normpath('./output'))
    local_directory = Path('./output').norm()
    print('Local directory = {}',format(local_directory))
    bucket = S3_BUCKET

    client = boto3.client('s3')

    # enumerate local files recursively
    for root, dirs, files in os.walk(local_directory):
        for filename in files:
            # construct the full local path
            local_path = Path(root, filename).norm()
            print('Local path = {}', format(local_path))
            # construct the full S3 path
            s3_path = '/'.join(local_directory.rel_path_to(local_path).components())[1:]
            print('Searching for path "{}" in "{}" for {}'.format(s3_path, bucket, local_path))
            try:
                file = client.head_object(Bucket=bucket, Key=s3_path)
                lt = dt.datetime.utcfromtimestamp(local_path.mtime())
                #print('Local time {} and external time {}'.format(lt,file['LastModified']))
                print("Path found on S3! Skipping {} which has {}...".format(s3_path, file['LastModified']))

                try:
                    client.delete_object(Bucket=bucket, Key=s3_path)
                    print("Uploading %s..." % s3_path)
                    mime_type = mimetypes.guess_type(local_path)
                    client.upload_file(local_path, bucket, s3_path, ExtraArgs={'ContentType': mime_type[0],
                                                                               'ACL': 'public-read'})
                except:
                    print("Unable to delete %s..." % s3_path)
            except:
                print("Uploading %s..." % s3_path)
                mime_type = mimetypes.guess_type(local_path)
                client.upload_file(local_path, bucket, s3_path, ExtraArgs={'ContentType': mime_type[0],
                                                                           'ACL': 'public-read'})
    # s3cmd sync $(OUTPUTDIR)/ s3://$(S3_BUCKET) --acl-public --delete-removed --guess-mime-type --no-mime-magic --no-preserve




@task
def gh_pages():
    """Publish to GitHub Pages"""
    rebuild()
    local("ghp-import -b {github_pages_branch} {deploy_path} -p".format(**env))




from fabric.api import task, env

# Need to modify path so fab_support is found
# This is only needed to run test fabfiles in a test directory
import sys
from pathlib import Path  # if you haven't already done so
file = Path(__file__).resolve()
parent, root = file.parent, file.parents[2]
sys.path.append(str(root))

# noinspection PyUnresolvedReferences
import fab_support
# noinspection PyUnresolvedReferences
import fab_support.django as django

# Definition of different environments to deploy to
env['stages'] = {
    'test': {
        'comment': 'Test version of Django to be deployed on Heroku',
        'DJANGO_SETTINGS_MODULE': 'demo_django.settings',  # Essential as django doesn't know name of app
        'HEROKU_APP_NAME': 'fab-support-test-postgres-test',
        'GIT_PUSH': 'git subtree push --prefix tests/demo_django_postgres heroku master',
        'GIT_PUSH_DIR': '../..',
        'GIT_BRANCH': 'master'
    },
    'uat': {
        'comment': 'UAT version of Django to be deployed on Heroku',
        'DJANGO_SETTINGS_MODULE': 'demo_django.settings',  # Essential as django doesn't know name of app
        'HEROKU_APP_NAME': 'fab-support-test-postgres-uat',
        'GIT_PUSH': 'git subtree push --prefix tests/demo_django_postgres heroku master',
        'GIT_PUSH_DIR': '../..',
        'GIT_BRANCH': 'master'
    },
    'prod': {
        'comment': 'Production version of Django to be deployed on Heroku',
        'DJANGO_SETTINGS_MODULE': 'demo_django.settings',  # Essential as django doesn't know name of app
        'HEROKU_APP_NAME': 'fab-support-test-postgres-prod',
        'GIT_PUSH': 'git subtree push --prefix tests/demo_django_postgres heroku master',
        'GIT_PUSH_DIR': '../..',
        'GIT_BRANCH': 'master'
    },
}


@task
def test_django_postgres_fab_file():
    """A test task to show correct file has been loaded"""
    pass
